`default_nettype none

module segment_driver
  ( input wire [3:0] VAL,
    output wire [6:0] CATHODE );

   assign CATHODE = (VAL==4'h0) ? 7'b000_0001 :
                    (VAL==4'h1) ? 7'b100_1111 :
                    (VAL==4'h2) ? 7'b001_0010 :
                    (VAL==4'h3) ? 7'b000_0110 :
                    (VAL==4'h4) ? 7'b100_1100 :
                    (VAL==4'h5) ? 7'b010_0100 :
                    (VAL==4'h6) ? 7'b010_0000 :
                    (VAL==4'h7) ? 7'b000_1111 :
                    (VAL==4'h8) ? 7'b000_0000 :
                    (VAL==4'h9) ? 7'b000_1100 :
                    (VAL==4'ha) ? 7'b000_1000 :
                    (VAL==4'hb) ? 7'b110_0000 :
                    (VAL==4'hc) ? 7'b111_0010 :
                    (VAL==4'hd) ? 7'b100_0010 :
                    (VAL==4'he) ? 7'b011_0000 :
                    (VAL==4'hf) ? 7'b011_1000 : 7'h0;
endmodule // segment_driver

module kurukuru
  ( input wire CLK, RST, 
    output reg [7:0] ANODE );
   
   parameter CounterBits = 10;

   reg [(CounterBits-1):0] CNT;
   wire                    STROBE = &CNT;

   always @ (posedge CLK) begin
      if (RST) begin
         CNT <= 0;
         ANODE <= 8'b0111_1111;
      end else begin
         CNT <= CNT+1;
         if (STROBE)
           ANODE <= {ANODE[0], ANODE[7:1]};
      end
   end
endmodule // kurukuru

module segment_disp_ctrl
  ( input wire CLK, RST,
    input wire [31:0] VAL,
    output wire [7:0] ANODE,
    output wire [7:0] CATHODE );

   parameter ColumnCounterBits=10;

   kurukuru # ( .CounterBits(ColumnCounterBits) )
   kuru (.CLK(CLK), .RST(RST), .ANODE(ANODE) );

   wire [3:0]    DISP_VAL;

   assign DISP_VAL = ( ~ANODE[7] ) ? VAL[31:28] :
                     ( ~ANODE[6] ) ? VAL[27:24] :
                     ( ~ANODE[5] ) ? VAL[23:20] :
                     ( ~ANODE[4] ) ? VAL[19:16] :
                     ( ~ANODE[3] ) ? VAL[15:12] :
                     ( ~ANODE[2] ) ? VAL[11: 8] :
                     ( ~ANODE[1] ) ? VAL[ 7: 4] : VAL[3:0];

   segment_driver sd ( .VAL(DISP_VAL), .CATHODE(CATHODE[6:0]) );

   assign CATHODE[7] = 1'b1;  // no dot is illuminated

endmodule // segment_disp_ctrl

`default_nettype wire
