`default_nettype none

module push_filter
  ( input wire CLK, RST,
    input wire I,
    output wire O );

   wire         DF_OUT;
   double_flop df ( .CLK(CLK), .I(I), .O(DF_OUT) );
   push_to_pulse p2p ( .CLK(CLK), .RST(RST), .I(DF_OUT), .O(O) );

endmodule // push_filter

module double_flop
  ( input wire CLK,
    input wire  I,
    output wire O );

   reg          R1, R2;
   always @ (posedge CLK) begin
      R1 <= I;
      R2 <= R1;
   end

   assign O = R2;

endmodule // double_flop

module push_to_pulse
  ( input wire CLK, RST,
    input wire  I,
    output wire O );

//    parameter CNTMax = 10 * 1000 * 1000; // 0.1s @ 100MHz
   parameter CNTMax =  100 * 1000; // 0.001s = 1ms @ 100MHz

   reg          PRESS, PRESS_R;
   reg [31:0]   CNT;

   always @ (posedge CLK) begin
      PRESS_R <= PRESS;
      if (RST) begin
         PRESS <= 0;
      end else begin
         if (~PRESS) begin
            if (I) begin
               PRESS <= 1;
               CNT <= 0;
            end
         end else begin
            CNT <= I ? 0 : CNT+1;
            if (CNT== CNTMax) PRESS <= 0;
         end
      end
   end

   assign O = PRESS & ~PRESS_R;

endmodule // push_to_pulse

`default_nettype wire
