`default_nettype none

module top
  ( input wire CLK, RST,
    input wire        BTNL, BTNR,
    output wire [7:0] ANODE,
    output wire [7:0] CATHODE,
    input wire        RXD, 
    output wire       TXD);

   wire               BRAM_CLK;
   wire [31:0]   BRAM_A, BRAM_DIN, BRAM_DOUT;
   wire [3:0]    BRAM_WE;
   wire          BRAM_EN;
   
   
   design_1_wrapper mb
     ( .BRAM_PORT_0_addr(BRAM_A),
       .BRAM_PORT_0_clk (BRAM_CLK),
       .BRAM_PORT_0_din (BRAM_DIN),  // O
       .BRAM_PORT_0_dout(BRAM_DOUT), // I
       .BRAM_PORT_0_en  (BRAM_EN),
       .BRAM_PORT_0_rst (),
       .BRAM_PORT_0_we  (BRAM_WE),

       .Clk(CLK),
       .reset_rtl_0(RST),
       .uart_rtl_0_rxd(RXD),
       .uart_rtl_0_txd(TXD)
       );

   // reg32w + segment_disp_ctrl

   wire [31:0]   SEG_VAL;
   
   reg32w reg_w
     ( .CLK(BRAM_CLK), .WE(&BRAM_WE),
       .A(BRAM_A), .DIN(BRAM_DIN),
       .VAL(SEG_VAL) );

   segment_disp_ctrl sd
     ( .CLK(BRAM_CLK), .RST(RST),
       .VAL(SEG_VAL),
       .ANODE(ANODE), .CATHODE(CATHODE) );

   // reg32r + button_count

   wire          INCL, INCR;
   
   push_filter pf_l
     ( .CLK(BRAM_CLK), .RST(RST), .I(BTNL), .O(INCL) );
   push_filter pf_r
     ( .CLK(BRAM_CLK), .RST(RST), .I(BTNR), .O(INCR) );

   reg [31:0]    CNTL, CNTR;
   always @ (posedge BRAM_CLK) begin
      if (RST) begin
         CNTL <= 0; CNTR <= 0;
      end else begin
         if (INCL) CNTL <= CNTL+1;
         if (INCR) CNTR <= CNTR+1;
      end
   end

   reg32r reg_r
     ( .CLK(BRAM_CLK), 
       .A(BRAM_A), .DOUT(BRAM_DOUT),
       .VAL1(CNTL), .VAL2(CNTR) );
   
endmodule // top

`default_nettype wire
