#include "xparameters.h"
#include "xil_exception.h"
#include "xstreamer.h"
#include "xil_cache.h"
#include "xllfifo.h"
#include "xstatus.h"


#define FIFO_DEV_ID XPAR_AXI_FIFO_0_DEVICE_ID
#define WORD_SIZE 4
#define NUM_WORDS 0x123

#undef DEBUG

XLlFifo FifoInstance;
u32 SourceBuffer[NUM_WORDS], DestinationBuffer[NUM_WORDS];

int init(XLlFifo *InstancePtr, u16 DeviceId){
  XLlFifo_Config *Config;
  int Status = XST_SUCCESS;
  int i;

  // Initialize the driver
  Config = XLlFfio_LookupConfig(DeviceId);
  if (!Config) {
    xil_printf("No config found for %d\r\n", DeviceId);
    return XST_FAILURE;
  }

  Status = XLlFifo_CfgInitialize(InstancePtr, Config, Config->BaseAddress);
  if (Status != XST_SUCCESS) {
    xil_printf("Initialization failed\n\r");
    return Status;
  }

  // Check reset value
  Status = XLlFifo_Status(InstancePtr);
  XLlFifo_IntClear(InstancePtr,0xffffffff);
  Status = XLlFifo_Status(InstancePtr);
  if(Status != 0x0) {
    xil_printf("\n ERROR : Reset value of ISR0 : 0x%x\t"
               "Expected : 0x0\n\r",
               XLlFifo_Status(InstancePtr));
    return XST_FAILURE;
  }
  return Status;
}

int send(XLlFifo *InstancePtr, u32 *src, u32 len){
  xil_printf(" Transmitting Data ... \r\n");

  for(int i=0; i<len; i++){
    if(XLlFifo_iTxVacancy(InstancePtr))
      XLlFifo_TxPutWord(InstancePtr, src[i]);
    else{
      xil_printf("Tx buffer overrun!\n\r");
      return XST_FAILURE;
    }
  }
        
  // start transmission
  XLlFifo_iTxSetLen(InstancePtr, len*WORD_SIZE);

  // wait for termination
  while( !(XLlFifo_IsTxDone(InstancePtr)) ){}

  return XST_SUCCESS;
}

int recv (XLlFifo *InstancePtr, u32* dest){
  int i;
  int Status;
  static u32 ReceiveLength;

  xil_printf(" Receiving data ....\n\r");

  // Read Recv length
  ReceiveLength = (XLlFifo_iRxGetLen(InstancePtr))/WORD_SIZE;
  xil_printf("Recv length %d\r\n", ReceiveLength);

  // Start receiving
  for ( i=0; i < ReceiveLength; i++){
    u32 RxWord = XLlFifo_RxGetWord(InstancePtr);

    if(XLlFifo_iRxOccupancy(InstancePtr)){
      RxWord = XLlFifo_RxGetWord(InstancePtr);
    }
    dest[i] = RxWord;
  }

  Status = XLlFifo_IsRxDone(InstancePtr);
  if(Status != TRUE){
    xil_printf("Failing in receive complete ... \r\n");
    return -1;
  }

  return ReceiveLength;
}

int main(){
  int Status;

  Status = init(&FifoInstance, FIFO_DEV_ID);
  if (Status != XST_SUCCESS) {
    xil_printf("Init failed\n\r");
    return XST_FAILURE;
  }

  // Prepare
  for(int i=0; i<NUM_WORDS; i++) SourceBuffer[i] = i;

  // Send
  Status = send(&FifoInstance, SourceBuffer, NUM_WORDS);
  if (Status != XST_SUCCESS){
    xil_printf("Transmisson of Data failed\n\r");
    return XST_FAILURE;
  }

  // Receive
  int len = recv(&FifoInstance, DestinationBuffer);
  if (len == -1){
    xil_printf("Receiving data failed\n\r");
    return XST_FAILURE;
  }

  // Show result
  for (int i=0; i<len; i++)
    xil_printf("Recv data[%d]: %x\n\r", i, DestinationBuffer[i]);

  xil_printf("Done.\n\r");

  return XST_SUCCESS;
}
