`default_nettype none

module top
  ( input wire CLK, RST,
    input wire        BTNL, BTNR,
    output wire [7:0] ANODE,
    output wire [7:0] CATHODE,
    input wire        RXD, 
    output wire       TXD);

   // BRAM mapped I/O signals
   wire               BRAM_CLK;
   wire [31:0]   BRAM_A, BRAM_DIN, BRAM_DOUT;
   wire [3:0]    BRAM_WE;
   wire          BRAM_EN;

   // AXI loopback signals
   wire          TREADY, TVALID, TLAST;
   wire [31:0]   TXDATA, RXDATA;

   assign RXDATA = {4{TXDATA[7:0]}};
   
   design_1_wrapper mb
     ( .BRAM_PORT_0_addr(BRAM_A),
       .BRAM_PORT_0_clk (BRAM_CLK),
       .BRAM_PORT_0_din (BRAM_DIN),  // O
       .BRAM_PORT_0_dout(BRAM_DOUT), // I
       .BRAM_PORT_0_en  (BRAM_EN),
       .BRAM_PORT_0_rst (),
       .BRAM_PORT_0_we  (BRAM_WE),

       .AXI_STR_RXD_0_tdata (RXDATA), //  I [31:0]
       .AXI_STR_RXD_0_tlast (TLAST),  // I
       .AXI_STR_RXD_0_tready(TREADY), // O
       .AXI_STR_RXD_0_tvalid(TVALID), // I
       .AXI_STR_TXD_0_tdata (TXDATA), // O [31:0]
       .AXI_STR_TXD_0_tlast (TLAST),  // O
       .AXI_STR_TXD_0_tready(TREADY), // I
       .AXI_STR_TXD_0_tvalid(TVALID), // O
       
       .Clk(CLK),
       .reset_rtl_0(RST),
       .uart_rtl_0_rxd(RXD),
       .uart_rtl_0_txd(TXD)
       );

   // AXI stream monitor + 7 segment display
   reg [31:0]    AXI_CNT;
   always @ (posedge CLK) begin
      if (RST) begin
         AXI_CNT <= 0;
      end else begin
         if (TVALID & TREADY) AXI_CNT <= AXI_CNT+1;
      end
   end

   segment_disp_ctrl sd
     ( .CLK(CLK), .RST(RST),
       .VAL(AXI_CNT),
       .ANODE(ANODE), .CATHODE(CATHODE) );

   // reg32r + button_count

   wire          INCL, INCR;
   
   push_filter pf_l
     ( .CLK(CLK), .RST(RST), .I(BTNL), .O(INCL) );
   push_filter pf_r
     ( .CLK(CLK), .RST(RST), .I(BTNR), .O(INCR) );

   reg [31:0]    CNTL, CNTR;
   always @ (posedge CLK) begin
      if (RST) begin
         CNTL <= 0; CNTR <= 0;
      end else begin
         if (INCL) CNTL <= CNTL+1;
         if (INCR) CNTR <= CNTR+1;
      end
   end

   reg32r reg_r
     ( .CLK(CLK), 
       .A(BRAM_A), .DOUT(BRAM_DOUT),
       .VAL1(CNTL), .VAL2(CNTR) );
   
endmodule // top

`default_nettype wire
